/* ---------------------------------------------------------- */
/* | Nombre: stermp.c ( Simple Terminal Play)               | */
/* | Autor:  Gaspar Fernández (blakeyes@totaki.com)         | */
/* | Fecha:  05 Enero 2009                                  | */
/* | Web:    http://www.totaki.com/poesiabinaria/           | */
/* ---------------------------------------------------------- */
/* | Descripción:                                           | */
/* |   Manejo de colores y posicionamiento de texto en un   | */
/* |  terminal para nostálgicos de conio.h                  | */
/* ---------------------------------------------------------- */
/*   Este código se distribuye libremente, tal cual, sin garantías. */
/*   Pero si utilizas este código, y lo distribuyes, me gustaría que */
/*   citaras la fuente, y ya de paso, un enlace al blog no vendría mal :) */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "stermp.h"

/* Equivalencias de colores */
static const char ansicolors[16] = {30, 34, 32, 36, 31, 35, 33, 37, 0, 34, 32, 36, 31, 35, 33, 37};

struct termios	save_tty;	/* Copia de seguridad de atributos de terminal */


void clrscr()			/* Borrar la pantalla */
{
  printf("\033[2J");   // Borra pantalla
  printf("\033[1;1H"); // Posicionamos el cursor en la fila 1 columna 1
}

void gotoxy(int x, int y)
{
  printf("\033[%d;%dH", y, x); 	/* Posicionamos el cursor en X, Y */
}

void textcolor(int color)
{
  int atrval=0;
  if (color & UNDERLINE)
    atrval=UNDERLINE_ATTR;
  else if (color & BLINK)
    atrval=BLINK_ATTR;
  else if (color>=BRILLO_MIN)
    atrval=BRILLO_ATTR;

  color=ansicolors[color & 15];		/* Eliminamos los atributos */
  
  printf("\033[%d;%dm", atrval, color);
}

void textbackground(int color)
{
  if (color<BRILLO_MIN)
    {				    /* Sólo ciertos colores tienen fondo */
      color=ansicolors[color & 15]+10;		/* Eliminamos los atributos */
  
      printf("\033[%dm", color);
    }
}

void restore_color()
{
  printf("\033[0m");
}

int wherex()
{
  coord_t coord;

  coord=wherexy(0);
  return coord.x;
}

int wherey()
{
  coord_t coords;

  coords=wherexy(0);
  return coords.y;
}

int screenwidth()
{
  coord_t coords;

  coords=wherexy(1);
  return coords.x;
}

int screenheight()
{
  coord_t coords;

  coords=wherexy(1);
  return coords.y;
}

coord_t wherexy(int abs)
{
  coord_t coords;
  char buf[12];
  char *p;
  termnoecho();

  fflush(stdout);		/* Actualizamos stdout porque a lo mejor  */

  if (abs)			/* Si queremos las coordenadas de la pantalla... */
    {
	fprintf(stderr, "\033[s");	  /* Salva la posición actual del cursor */
	fprintf(stderr, "\033[r");	  /* Elimina el scroll */
	fprintf(stderr, "\033[255;255H"); /* Nos vamos a la esquina inferior derecha */
    }

  fprintf(stderr, "\033[6n");

  while (read(STDIN_FILENO, buf, sizeof(buf))==0);
  
  sscanf(buf, "\033[%d;%dR", &coords.y, &coords.x);

  if (abs)
    {
	fprintf(stderr, "\033[u"); /* Restauramos la posición del cursor */
    }

  restore_terminal();		/* Restauramos los atributos del terminal */

  return coords;
}

void termnoecho()
{
  struct termios   tty;
  int		   save_tty_valid = 0;

  /* Copia de seguridad de atributos del terminal */
  if ( tcgetattr(STDIN_FILENO, &save_tty) != 0 )
    {
      fprintf(stderr, "ERROR: No puedo obtener atributos del terminal\n");
      exit(1);
    }
  
  save_tty_valid = 1;
  
  tty = save_tty;
  
  /* Desactivamos atributos del terminal */
  tty.c_lflag &= ~(ICANON);	
  tty.c_lflag &= ~(ECHO|ECHOE|ECHOK|ECHONL);
  tty.c_cc[VMIN]  = 1;
  tty.c_cc[VTIME] = 1;
  
  if ( tcsetattr(STDIN_FILENO, TCSANOW, &tty) != 0 )
    {
      fprintf(stderr, "ERROR: cannot definir atributos al terminal\n");
      exit(1);
    } 
}

void restore_terminal()
{
  if ( tcsetattr(STDIN_FILENO, TCSANOW, &save_tty) != 0 )
    {
      fprintf(stderr, "ERROR: cannot definir atributos al terminal\n");
      exit(1);
    } 
}
